#!/bin/sh
#------------------------------------------------------------------------------
# status.cgi - Display Status of ISDN & DSL
#
# Creation:     17.01.2001  tg
# Last Update:  $Id: status.cgi 10822 2006-01-11 18:35:42Z hh $
#
#------------------------------------------------------------------------------
# get main helper functions
. /srv/www/include/cgi-helper

# Functions
get_dns_name ()
{
    set -- `nslookup $1 | grep ^Name || echo "dummy $1"`
    echo $2
}

ip_to_dnsname ()
{   # lookup unkown IPs
    for ip in $list
    do
        echo "s/>$ip</ title=\"$ip\">`get_dns_name $ip`</g" >> /tmp/sed.$$
    done
}

SEC_ACTION='view'

case $FORM_action in
    "")
        headtitle="$_connstatus"
    ;;
    *)
        ref="refresh=1;"
        case $FORM_action in
            "$_STAT_reboot"|"$_STAT_halt")
                SEC_ACTION='boot'
                ref=""
            ;;
            dialmode*)
                SEC_ACTION='dialmode'
            ;;
            circuit*)
                SEC_ACTION='circuit'
            ;;
            "$_STAT_dial"|"$_STAT_hangup")
                SEC_ACTION='dial'
                eval 'headtitle=$_'$FORM_action
            ;;
            "$_STAT_addlink"|"$_STAT_remlink") SEC_ACTION='link'
                eval 'headtitle=$_'$FORM_action
            ;;
        esac
    ;;
esac

# Security
check_rights "status" "$SEC_ACTION"

# Check if imond is running
if [ ! -f /var/run/imond.port ]
then
    show_html_header "$_MN_err"
    show_error "$_MN_err" "Imond $_MN_notrun.<br>$_MN_rbfix"
    show_html_footer
    exit 1
fi

show_html_header "$headtitle" "${ref}url=$myname"

port=`cat /var/run/imond.port`

case "$FORM_action" in
    "$_STAT_hangup")
        show_info "" "$_STAT_hangingup ... `{ echo "hangup"; echo "quit"; } | netcat localhost $port`"
    ;;
    "$_STAT_dial")
        show_info "" "$_STAT_dialing ... `{ echo "dial"; echo "quit"; } | netcat localhost $port`"
    ;;
    "$_STAT_addlink")
        route=`{ echo "route"; echo "quit"; } | netcat localhost $port | sed 's/OK //; s/.$//'`
        show_info "" "$_STAT_addinglink ... `{ echo "addlink $route"; echo "quit"; } | netcat localhost $port`"
    ;;
    "$_STAT_remlink")
        route=`{ echo "route"; echo "quit"; } | netcat localhost $port | sed 's/OK //; s/.$//'`
        show_info "" "$_STAT_removinglink ... `{ echo "removelink $route"; echo "quit"; } | netcat localhost $port`"
    ;;

    circuit)
        show_info "" "$_STAT_changingcirc $FORM_circuit ... `{ echo "route $FORM_circuit" ; echo "quit"; } | netcat localhost $port`"
    ;;
    dialmode)
        show_info "" "$_STAT_changingdialmode \"$FORM_dialmode\" ...`{ echo "dialmode $FORM_dialmode" ; echo "quit"; } | netcat localhost $port`"
    ;;
    "$_STAT_reboot")
        show_info "" "$_STAT_rebooting ... `{ echo "reboot" ; echo "quit"; } | netcat localhost $port`"
    ;;
    "$_STAT_halt")
        show_info "" "$_STAT_halting ... `{ echo "halt" ; echo "quit"; } | netcat localhost $port`"
  ;;


"")

set -- `{ echo "channels"; echo "pppoe"; echo "cpu"; echo "quit"; } | netcat localhost $port |
  sed 's/\(OK\|ERR\) //; s/.$//'`

pppoe="$2"
usage="$3"
chan="$1"

#---------------------------------------------------------------------------------------------
#------------------------- CPU-USAGE ---------------------------------------------------------
#---------------------------------------------------------------------------------------------
show_tab_header "$_STAT_cpuusage" no

echo "<table class=\"normtable\"><tr><td>$_STAT_cpuusage: </td><td>"
echo "<form action=\"$myname\" method=\"get\" class=\"usagetext\"><input type=\"Text\" name=\"cpuusage\" id=\"cpuusage\" value=\"$usage %\" size=\"6\" readonly></form>"
echo "</td><td id=\"cpu1\" width=\"$usage\"></td>"
echo "<td id=\"cpu2\" width=\"`expr 100 - $usage`\"></td>"

echo '</tr></table>'
show_tab_footer

echo "<br>"
#---------------------------------------------------------------------------------------------
#------------------------- Last Call ---------------------------------------------------------
#---------------------------------------------------------------------------------------------

# Check if telmond is running
TELMOND_PORT="`ps ax | grep telmond | grep -v grep |sed -n 's/^[^!]*-port \([^ ]*\)[^!]*/\1/p'`"
case $TELMOND_PORT in
    "") ;;
    *)
        LASTCALL=`netcat localhost $TELMOND_PORT 2>/dev/null | sed 's/.$//'`

        case $LASTCALL in
            "") ;;
            *)
                LASTTIME=`echo $LASTCALL | cut -f 1-2 -d" "`
                LASTFROM=`echo $LASTCALL | cut -f 3 -d" "`
                LASTTO=`echo $LASTCALL | cut -f 4 -d" "`

                # Find names in phonebook
                NAME=`grep "^$LASTFROM=" /etc/phonebook | cut -d"=" -f 2- | cut -d"," -f 1`
                if [ "$NAME" != "" ]; then LASTFROM="$NAME"; fi
                NAME=`grep "^$LASTTO=" /etc/phonebook | cut -d"=" -f 2- | cut -d"," -f 1`
                if [ "$NAME" != "" ]; then LASTTO="$NAME"; fi

                show_tab_header "$_STAT_lastcall" no
                echo '<table class="normtable">'
                echo '<tr><th rowspan="2">'$_STAT_lastcall':</th>'
                echo '<th>'$_STAT_from'</th><th>'$_STAT_to'</th>'
                echo '<th>'$_MN_date' / '$_MN_time'</th></tr><tr>'
                echo "<td>$LASTFROM</td><td>$LASTTO</td>"
                echo "<td>$LASTTIME</td></tr></table>"
                show_tab_footer
            ;;
        esac
    ;;
esac
echo "<br>"

#---------------------------------------------------------------------------------------------
#------------------------- Connection-Status -------------------------------------------------
#---------------------------------------------------------------------------------------------

onl=0
devs=''
for i in `seq 1 $chan`
do
  devs="$devs $i"
done
case $pppoe in
    1) devs="pppoe $devs" ;;
esac

# check if there are devices to display
case $devs in
    "") ;;
    *)
        show_tab_header "$_STAT_connectionstatus" no
        echo '<table class="normtable">'
        echo "<tr><th>$_STAT_state</th><th>$_STAT_name</th><th>$_STAT_direction</th><th>$_STAT_ip</th>
              <th>$_STAT_ibytes</th><th>$_STAT_obytes</th><th>$_STAT_onltime</th></tr>"

        for i in $devs
        do
            cname=
            cdir=
            cip=
            conltime=
            cirate=
            corate=
            cibytes=
            cobytes=
            ctime=
            chtime=
            charge=
            conninfo=`{ echo "status $i"; echo "phone $i"; echo "inout $i"; echo "ip $i"
                        echo "online-time $i"; echo "rate $i"; echo "quantity $i"
                        echo "time $i"; echo "chargetime $i"; echo "charge $i"; echo "quit"
                      } | netcat localhost $port | sed 's/OK //; s/.$//'`
            set -- $conninfo
            case $1 in
                Off*)
                    cimg=rotaus
                    #cibytes=`bytes2read $7 $8`B
                    #cobytes=`bytes2read $9 ${10}`B
                    ctime=${12}
                    chtime=${13}
                    charge=${14}
                ;;
                Call*)
                    cimg=gelbaus
                    cname=$2
                    #cibytes=`bytes2read $7 $8`B
                    #cobytes=`bytes2read $9 ${10}`B
                    ctime=${12}
                    chtime=${13}
                    charge=${14}
                ;;
                On*)
                    cimg=gruenaus
                    cname=$2
                    eval cdir='$_STAT_'$3
                    cip=$4
                    conltime=$5
                    cirate=`bytes2read 0 $6`B/s
                    corate=`bytes2read 0 $7`B/s
                    cibytes=`bytes2read $8 $9`B
                    cobytes=`bytes2read ${10} ${11}`B
                    ctime=${12}
                    chtime=${13}
                    charge=${14}
                    onl=1
                ;;
            esac
            form='<form action="$myname" method="GET" class="connectiontext">'
            echo '<tr>'
            echo '  <td><img id="cimg'$i'" src="../img/'$cimg'.gif" width="16" height="16" alt=""></td>'
            echo '  <td>'$form'<input type="Text" id="cname'$i'" value="'$cname'" size="15" readonly></form></td>'
            echo '  <td>'$form'<input type="Text" id="cdir'$i'" value="'$cdir'" size="10" readonly></form></td>'
            echo '  <td>'$form'<input type="Text" id="cip'$i'" value="'$cip'" size="15" readonly></form></td>'
            #echo '  <td>'$form'<input type="Text" id="cirate'$i'" value="'$cirate'" size="6" readonly></form></td>'
            #echo '  <td>'$form'<input type="Text" id="corate'$i'" value="'$corate'" size="6" readonly></form></td>'
            echo '  <td>'$form'<input type="Text" id="cibytes'$i'" value="'$cibytes'" size="9" readonly></form></td>'
            echo '  <td>'$form'<input type="Text" id="cobytes'$i'" value="'$cobytes'" size="9" readonly></form></td>'
            echo '  <td>'$form'<input type="Text" id="conltime'$i'" value="'$conltime'" size="8" readonly></form></td>'
            #echo '  <td>'$form'<input type="Text" id="ctime'$i'" value="'$ctime'" size="6" readonly></form></td>'
            #echo '  <td>'$form'<input type="Text" id="chtime'$i'" value="'$chtime'" size="6" readonly></form></td>'
            #echo '  <td>'$form'<input type="Text" id="charge'$i'" value="'$charge'" size="6" readonly></form></td>'
            echo '</tr>'
        done

        echo "</table>"
        show_tab_footer
        echo "<br>"
    ;;
esac
#---------------------------------------------------------------------------------------------
#------------------------- Actions -----------------------------------------------------------
#---------------------------------------------------------------------------------------------

# Check for channel-bundling
route=`{ echo "route"; echo "quit"; } | netcat localhost $port | sed 's/OK //; s/.$//'`
links=`{ echo "links $route"; echo "quit"; } | netcat localhost $port | sed 's/OK //; s/.$//'`

show_tab_header "$_STAT_actions" no

# Circuits
circuits=`{ echo "circuits" ; echo "quit"; } | netcat localhost $port | sed 's/.$//; s/OK //'`
# check if a circuit is defined
case $circuits in
    0);;
    *)
        comm=''
        for i in `seq 1 $circuits`
        do
          comm=`if [ "$comm" != "" ]; then echo "$comm"; fi; echo "circuit $i"`
        done
        circuits=`{ echo "$comm"; echo "quit"; } | netcat localhost $port |
          grep -n 'OK' | sed 's/^/<li><a href="'$myname'?action=circuit\&amp;circuit=/; s/:OK /">/;
          s/.$/<\/a><\/li>/'`
        circuits="<li><a href=\"$myname?action=circuit&amp;circuit=0\">Automatic (LCR)</a></li>
        $circuits"
        aktcirc=$route
        circuits=`echo "$circuits" | sed '/circuit='$aktcirc'/ s/<li/<li id="cselected"/'`
        echo "<ul class=\"circuits\"><li>$_STAT_circuits:</li>$circuits</ul>"

        # Dialmode
        dialmode="<li><a href=\"$myname?action=dialmode&amp;dialmode=auto\">Auto</a></li>
          <li><a href=\"$myname?action=dialmode&amp;dialmode=manual\">Manual</a></li>
          <li><a href=\"$myname?action=dialmode&amp;dialmode=off\">Off</a></li>"
        aktdialm=`{ echo "dialmode"; echo "quit"; } | netcat localhost $port | sed 's/OK //; s/.$//'`
        dialmode=`echo "$dialmode" | sed '/dialmode='$aktdialm'/ s/<li/<li id="dselected"/'`
        echo "<ul class=\"dialmode\"><li>$_STAT_dialmode:</li>$dialmode</ul>"

        # Dial/ Hangup
        echo '<form action="'$myname'" method="GET">'
        echo -n '<input id="dial" type="submit"'
        case $onl in
            1) echo -n ' value="'$_STAT_hangup'" name="action" ' ;;
            *) echo -n ' value="'$_STAT_dial'" name="action"' ;;
        esac
        echo ' class="actions">'

        # Channel-Bundling?
        case $links in
            0|ERR) ;;
            *)
                echo '<br>'
                echo -n '<input type="submit"'
                case "$links" in
                    1) echo -n ' value="'$_STAT_addlink'" name="action"' ;;
                    2) echo -n ' value="'$_STAT_remlink'" name="action"' ;;
                esac
                echo 'class="actions">'
            ;;
        esac
    ;;
esac

# Halt/Reboot
echo '<input type="submit" value="'$_STAT_reboot'" name="action" class="actions">'
echo '<input type="submit" value="'$_STAT_halt'" name="action" class="actions">'
echo '</form>'
show_tab_footer
echo '<br style="clear: both">'

#---------------------------------------------------------------------------------------------
#------------------------- Online Hosts -------------------------------------------------
#---------------------------------------------------------------------------------------------
show_tab_header "Online Hosts" no

echo '<table class="normtable"><tr><th>Status</th><th>Name</th></tr>'

cat >/tmp/sed1.$$ <<EOF
/^IP address/d
s/[[:space:]]\+/~/g
s/^/~/g
/~0x2~/s/^/online/g
EOF
cat >/tmp/sed.$$ <<EOF
s/^online~/<tr class="online"><td>online<\/td><td>/g
s/^~/<tr class="offline"><td>offline<\/td><td>/g
s/$/<\/td><\/tr>/g
EOF

sed -f /tmp/sed1.$$ /proc/net/arp | cut -d~ -f1-2 > /tmp/arp.$$
list=`cat /tmp/arp.$$ | cut -d~ -f2`
ip_to_dnsname
sed -f /tmp/sed.$$ /tmp/arp.$$

rm /tmp/sed.$$ /tmp/sed1.$$ /tmp/arp.$$

echo "</table>"
show_tab_footer

echo '<div id="refresh">'
#echo "$_MN_refresh ..."
echo "<iframe src=\"status_refresh.cgi\" class=\"HiddenFrame\">"
echo '</iframe>'
echo '</div>'


show_html_footer
;;
esac
